/*****************************************************************************
 *
 * FILE:	netfd.h
 * DESCRIPTION:	Easy Network I/O library header file
 * DATE:	Sat, Jun 10 2006
 * UPDATED:	Mon, Sep 17 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2006-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2006-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: netfd.h,v 1.11 2007/09/17 13:03:33 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_NETFD_H
#define	_NETFD_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <net/bpf.h>
#include <net/ethernet.h>
#include <netdb.h>

/******************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	NETFD_VERSION_NUMBER	0x10200000L
#define	NETFD_VERSION		"NetFD/1.2"
#define	NETFD_VERSION_TEXT	NETFD_VERSION " (2007/09/17)"
#define	NETFD_VERSION_TEXT_LONG \
	"Easy Network I/O Library 1.2, Mon, Sep 17 2007"

#ifndef	_BOOL_T
#define	_BOOL_T
typedef enum {
  false = 0,
  true	= 1
} bool;
#endif	/* _BOOL_T */

#define	NETFD_ETHER_ADDR_SIZE	(sizeof("ff:ff:ff:ff:ff:ff"))
#define	NETFD_IP_ADDR_SIZE	(sizeof("255.255.255.255"))

/* Easy Network I/O structure */
typedef struct _netfd_t {
#define	netfd_fileno(x)		((x)->osfd)
#define	netfd_addrlen(x)	((x)->addrlen)
#define	netfd_sockaddr(x)	((x)->saddr)
  int		osfd;		/* underlying OS file descriptor */
#define	NETFD_BUFSIZE	(1024*4)
  struct {		/* read buffer */
    int		total;	/* initialize to 0 */
    char *	bufp;	/* initialize to buf */
    char	buf[NETFD_BUFSIZE];
  } rb;
  char *		path;		/* path of unix domain socket */
  socklen_t		addrlen;	/* length of socket-address */
  struct sockaddr *	saddr;		/* socket-address for socket */
#define	NETFD_FL_LISTENER	0x0001	/* must call accept(2) */
  unsigned short	flags;
  struct _netfd_t *	next;
#if	DEBUG
  unsigned int		count;
#endif	/* DEBUG */
} netfd_t;
typedef netfd_t *	NetFD;

#define	NETFD_INIT_ERROR	(-1)
#define	NETFD_INIT_OK		(0)
#define	NETFD_INIT_ALREADY	(1)

#define	NETFD_MODEL_TINY	(8)
#define	NETFD_MODEL_MINI	(16)
#define	NETFD_MODEL_SMALL	(64)
#define	NETFD_MODEL_MEDIUM	(256)
#define	NETFD_MODEL_LARGE	(2048)
#define	NETFD_MODEL_HUGE	(16384)
#define	NETFD_DEFAULT_MODEL	NETFD_MODEL_SMALL

/* poll structure */
typedef struct {
  NetFD	nfd;		/* which netfd pointer to poll */
  short	events;		/* events we are interested in */
  short	revents;	/* events found on return */
} pollfd_t;
#define	NETFD_POLLIN	0x0001
#define	NETFD_POLLOUT	0x0002
#define	NETFD_POLLNVAL	0x0010

/* macro for timeout */
#define	NETFD_NO_TIMEOUT	(-1)
#define	NETFD_MSEC2SEC(msec)	((msec) / 1000)
#define	NETFD_MSEC2USEC(msec)	(((msec) % 1000) * 1000)
#define	NETFD_MSEC2NSEC(msec)	(((msec) % 1000) * 1000000)

/* macro for netfd_seek() */
#define	NETFD_SEEK_SET	SEEK_SET
#define	NETFD_SEEK_CUR	SEEK_CUR
#define	NETFD_SEEK_END	SEEK_END

/* macro for netfd_open_bpf() */
#define	NETFD_BPF_IMMEDIATE	0x0001	/* see BICIMMEDIATE in bpf(4) */
#define	NETFD_BPF_HDRCMPLT	0x0002	/* see BIOCSHDRCMPLT in bpf(4) */
#define	NETFD_BPF_NOSEESENT	0x0003	/* see BIOCSSEESENT in bpf(4) */

/* callbacks */
typedef void	(*netfd_callback_t)(NetFD nfd);
typedef int	(*netfd_icmp_callback_t)(
			uint8_t *		buf,
			ssize_t			buf_len,
			uint16_t		checked_icmp_id,
			struct sockaddr *	from,
			struct timeval *	recv_tv);

/*
 * Definition macro functions
 */
#define	netfd_recv(n,b,l,f,t)	netfd_recvfrom((n),(b),(l),(f),NULL,NULL,(t))
#define	netfd_send(n,m,l,f,t)	netfd_sendto((n),(m),(l),(f),NULL,0,(t))

#define	netfd_read_event_add(n) \
	netfd_event_set((n),EVFILT_READ,EV_ADD)
#define	netfd_read_event_del(n) \
	netfd_event_set((n),EVFILT_READ,EV_DELETE)
#define	netfd_write_event_add(n) \
	netfd_event_set((n),EVFILT_WRITE,EV_ADD)
#define	netfd_write_event_del(n) \
	netfd_event_set((n),EVFILT_WRITE,EV_DELETE)

/*
 * Interface information
 */
#define	IFI_NAME	(16)	/* same as IFNAMSIZ in <net/if.h> */
#define	IFI_HADDR	(8)	/* allow for 64-bit EUI-64 in future */
typedef struct ifi_info {
  uint8_t		ifi_name[IFI_NAME];	/* interface name */
  uint8_t		ifi_haddr[IFI_HADDR];	/* hardware address */
  uint16_t		ifi_hlen;	/* #bytes in hardware address */
  uint16_t		ifi_flags;	/* IFF_xxx constants from <net/if.h> */
  uint16_t		ifi_myflags;	/* our own IFI_xxx flags */
  struct sockaddr *	ifi_addr;	/* primary address */
  struct sockaddr *	ifi_brdaddr;	/* broadcast address */
  struct sockaddr *	ifi_dstaddr;	/* destination address */
  struct ifi_info *	ifi_next;	/* next of these structures */
} netfd_ifinfo_t;
#define	IFI_ALIAS	(1)	/* ifi_addr is an alias */

/******************************************************************************
 *
 *	Global functions declaration
 *
 *****************************************************************************/
int	netfd_init(int maxfds);
NetFD	netfd_open(const char * path, int flags, mode_t mode);
NetFD	netfd_fdopen(int osfd);
int	netfd_close(NetFD nfd);
off_t	netfd_seek(NetFD nfd, off_t offset, int whence);
ssize_t	netfd_read(NetFD nfd, void * buf, size_t nbyte, const time_t timeout);
ssize_t	netfd_write(NetFD nfd, const void * buf, size_t nbyte,
		    const time_t timeout);
ssize_t	netfd_readline(NetFD nfd, void * buf, size_t maxlen,
		       const time_t timeout);
ssize_t	netfd_readv(NetFD nfd, const struct iovec * iov, int iovcnt,
		    const time_t timeout);
ssize_t	netfd_writev(NetFD nfd, const struct iovec * iov, int iovcnt,
		     const time_t timeout);
ssize_t	netfd_recvfrom(NetFD nfd, void * buf, size_t len, int flags,
		       struct sockaddr * from, socklen_t * fromlen,
		       const time_t timeout);
ssize_t	netfd_sendto(NetFD nfd, const void * msg, size_t len, int flags,
		     const struct sockaddr * to, socklen_t tolen,
		     const time_t timeout);
NetFD	netfd_mkstemp(char * template);
#if	HAVE_FREEBSD_SENDFILE
int	netfd_sendfile(NetFD nfd, NetFD nsd, off_t init_offset, size_t nbytes,
		       struct sf_hdtr * hdtr, off_t * ret_sbytes, int flags,
		       const time_t timeout);
#elif	HAVE_LINUX_SENDFILE
int	netfd_sendfile(NetFD nfd, NetFD nsd, off_t init_offset, size_t nbytes,
		       void * hdtr, off_t * ret_sbytes, int flags,
		       const time_t timeout);
#else
int	netfd_sendfile(NetFD nfd, NetFD nsd, off_t offset, size_t nbytes,
		       void * hdtr, off_t * sbytes, int flags,
		       const time_t timeout);
#endif
NetFD	netfd_socket(int domain, int type, int protocol);
NetFD	netfd_accept(NetFD nfd, struct sockaddr * addr, socklen_t * addrlen,
		     const time_t timeout);
int	netfd_poll(pollfd_t nfds[], size_t nfdnum, const time_t timeout);
NetFD	netfd_tcp_client(const char * hostname, const char * service,
			 const time_t timeout);
NetFD	netfd_tcp_server(const char * hostname, const char * service,
			 const int backlog);
NetFD	netfd_udp_client(const char * hostname, const char * service,
			 bool broadcast);
NetFD	netfd_udp_connect(const char * hostname, const char * service);
NetFD	netfd_udp_server(const char * hostname, const char * service);
NetFD	netfd_udp_open2(const char * hostname, const char * service,
			const char * waitport, int family, bool broadcast,
			struct sockaddr ** to, socklen_t * tolen);
NetFD	netfd_ust_client(const char * path);
NetFD	netfd_ust_server(const char * path, const int backlog);
NetFD	netfd_udg_client(const char * path);
NetFD	netfd_udg_server(const char * path);
int	netfd_host_serv(const char * hostname, const char * service,
			int family, int socktype, int protocol,
			struct addrinfo ** res);
int	netfd_getnameinfo(NetFD nfd, char * host, size_t hostlen,
			  char * serv, size_t servlen);
char *	netfd_sock_ntop(const struct sockaddr * sa, bool with_port,
			char * dst, size_t size);
netfd_ifinfo_t *netfd_get_ifinfo(void);
void	netfd_free_ifinfo(netfd_ifinfo_t * ifihead);
int	netfd_dispatch(netfd_callback_t callback, bool writeshot,
		       const time_t timeout);
void	netfd_event_set(NetFD nfd, int filter, int flags);
/* ICMP */
NetFD	netfd_icmp_open(void);
ssize_t	netfd_icmp_send(NetFD nfd, const struct sockaddr * to,
			socklen_t tolen, uint16_t send_id,
			uint32_t seqno, const time_t timeout);
int	netfd_icmp_recv(NetFD nfd, struct sockaddr * from, socklen_t * fromlen,
			uint16_t check_id, const time_t timeout,
			netfd_icmp_callback_t callback);
/* Raw IP and BPF */
NetFD	netfd_open_raw_ip(bool has_hincl);
NetFD	netfd_open_bpf(const char * netif, uint16_t flags, size_t capsize,
		       bool promiscuous, const time_t to_ms,
		       struct bpf_program * pktfilter);
ssize_t	netfd_build_tcp_packet(const char * src_addr, uint16_t src_port,
			       const char * dst_addr, uint16_t dst_port,
			       uint8_t flags, uint32_t seqno,
			       uint32_t ackno, uint16_t wsize,
			       uint8_t ip_id, uint8_t ** packet);
ssize_t	netfd_build_udp_packet(const char * src_addr, uint16_t src_port,
			       const char * dst_addr, uint16_t dst_port,
			       uint8_t ip_id, uint8_t ** packet);
ssize_t	netfd_build_ip_packet(uint8_t protocol, const char * src_addr,
			      const char * dst_addr, uint8_t * data,
			      size_t datasize, uint8_t ip_ttl,
			      uint16_t ip_off, uint8_t ip_tos,
			      uint8_t ip_id, uint8_t ** packet);
ssize_t	netfd_build_ether_frame(uint16_t type, const char * src_host,
				const char * dst_host, uint8_t * data,
				size_t datasize, uint8_t ** frame);

#endif	/* _NETFD_H */
